## Copyright 2022 The IREE Authors
#
# Licensed under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""Generates all benchmarks."""

from typing import List, Tuple
from e2e_test_framework.definitions import iree_definitions
from benchmark_suites.iree import (riscv_benchmarks, x86_64_benchmarks,
                                   adreno_benchmarks, armv8_a_benchmarks,
                                   cuda_benchmarks, mali_benchmarks,
                                   vmvx_benchmarks)


def generate_benchmarks(
) -> Tuple[List[iree_definitions.ModuleGenerationConfig],
           List[iree_definitions.E2EModelRunConfig]]:
  benchmarks = [
      x86_64_benchmarks.Linux_x86_64_Benchmarks(),
      cuda_benchmarks.Linux_CUDA_Benchmarks(),
      riscv_benchmarks.Linux_RV64_Benchmarks(),
      riscv_benchmarks.Linux_RV32_Benchmarks(),
      armv8_a_benchmarks.Android_ARMv8_A_Benchmarks(),
      adreno_benchmarks.Android_Adreno_Benchmarks(),
      mali_benchmarks.Android_Mali_Benchmarks(),
      vmvx_benchmarks.Android_VMVX_Benchmarks()
  ]
  all_module_generation_configs = []
  all_run_configs = []
  for benchmark in benchmarks:
    module_generation_configs, run_configs = benchmark.generate()
    all_module_generation_configs += module_generation_configs
    all_run_configs += run_configs

  return (all_module_generation_configs, all_run_configs)
